/*
 * Common viewport-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __VIEWPORT_H__
#define __VIEWPORT_H__

#include "display.h"



/* An abstraction providing a scrolling viewport. */

typedef struct
{
    /* Display configuration. */

    display_config_t *cfg;

    /* Scrolling positions for each frame. */

    int *xorigins, *yorigins;

    /* Scrolling position. */

    int xorigin, yorigin;

    /* Scaling factors, translating the origin coordinates to display columns
       and rows. */

    int xscale, yscale;

    /* Update function after scrolling. */

    void (*update)();

} viewport_t;



/* Initialise a viewport.

   Viewport(<name>, display_config_t *cfg, int xscale, int yscale, void (*update)())
*/

#define Viewport(NAME, CFG, XSCALE, YSCALE, UPDATE) \
    int __##NAME##_xorigins[(CFG)->frames];         \
    int __##NAME##_yorigins[(CFG)->frames];         \
    viewport_t NAME = {                             \
        .cfg=CFG,                                   \
        .xorigins=__##NAME##_xorigins,              \
        .yorigins=__##NAME##_yorigins,              \
        .xorigin=0,                                 \
        .yorigin=0,                                 \
        .xscale=XSCALE,                             \
        .yscale=YSCALE,                             \
        .update=UPDATE};

/* Viewport operations. */

void viewport_set_origin(viewport_t *v, int xorigin, int yorigin);

void viewport_update_origin(viewport_t *v, int dx, int dy);

#endif /* __VIEWPORT_H__ */
